﻿using System;
using System.Windows;
using System.Windows.Media;

using Telerik.Windows.Controls;
using Telerik.Windows.Diagrams.Core;

namespace RDConnectionGradient
{
	public class GradientConnection : RadDiagramConnection
	{
		private readonly LinearGradientBrush grad;

		private readonly Color startColor;
		private readonly Color endColor;
		public GradientConnection()
		{
			this.grad = Application.Current.Resources["Grad"] as LinearGradientBrush; //not that you the start/end are frozen and cannot be reused
			if (this.grad == null) throw new Exception("Connection gradient template not found.");
			this.startColor = this.grad.GradientStops[0].Color;
			this.endColor = this.grad.GradientStops[1].Color;
			var startStop = new GradientStop(this.startColor, 0);
			var endStop = new GradientStop(this.endColor, 1);
			this.grad = new LinearGradientBrush(new GradientStopCollection(new[] { startStop, endStop }));
		}

		protected override void Update(bool isManipulating = false)
		{
			base.Update(isManipulating);
			var b = this.StartPoint;
			var e = this.EndPoint;
			var sx = (e.X - b.X);
			var sy = (e.Y - b.Y);
			var startPoint = new Point(0, 0);
			var endPoint = new Point(0, 0);
			if (Math.Abs(sx) < Utils.Epsilon) //vertical
			{
				if (Math.Abs(sy) < Utils.Epsilon) // as good as zero length
				{
					//  doesn't matter as the visual is as good as epsilon-like
				}
				else if (sy > 0) //N->S
				{
					startPoint = new Point(0, 0);
					endPoint = new Point(0, 1);
				}
				else //S->N
				{
					startPoint = new Point(0, 1);
					endPoint = new Point(0, 0);
				}

			}
			else if (sx > 0) // NE or SE
			{
				if (Math.Abs(sy) < Utils.Epsilon) // horizontal E->W
				{
					startPoint = new Point(0, 0);
					endPoint = new Point(1, 0);
				}
				else if (sy > 0) //NE->SW
				{
					startPoint = new Point(0, 0);
					endPoint = new Point(1, 1);
				}
				else //SE->NW
				{
					startPoint = new Point(0, 1);
					endPoint = new Point(1, 0);

				}
			}
			else // NW or SW
			{
				if (Math.Abs(sy) < Utils.Epsilon) // horizontal W->E
				{
					startPoint = new Point(1, 0);
					endPoint = new Point(0, 0);
				}
				else if (sy > 0) //NW->SE
				{
					startPoint = new Point(1, 0);
					endPoint = new Point(0, 1);
				}
				else //SW->NE
				{
					startPoint = new Point(1, 1);
					endPoint = new Point(0, 0);
				}
			}

			this.grad.StartPoint = startPoint;
			this.grad.EndPoint = endPoint;
			this.Stroke = this.grad;
		}
	}
}